/******************************************************************************************
 *
 * CHECK_IMAP.C
 *
 * Program: IMAP4 plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Tom Shields (tom.shields@basswood.com)
 *
 * Last Modified: 04-13-1999
 *
 * Command line: CHECK_IMAP <host_address> [port]
 *
 * Description:
 *
 * This plugin will attempt to open an IMAP connection with the host.  Successul connects 
 * return STATE_OK, refusals and timeouts return STATE_CRITICAL, other errors return
 * STATE_UNKNOWN.  Successful connects, but incorrect reponse messages from the host 
 * result in STATE_WARNING return values.
 *****************************************************************************************/

#include "../common/config.h"
#include <stdio.h>		/* obligatory includes */
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <time.h>

#define	STATE_CRITICAL 2	/* service state return codes */
#define STATE_WARNING  1
#define STATE_OK       0
#define STATE_UNKNOWN  -1

#define IMAP_PORT	110
#define IMAP_PROTO	"tcp"
#define IMAP_EXPECT1     "+OK"
#define IMAP_QUIT	"a1 LOGOUT\n"

#define MAX_INPUT_BUFFER	256

time_t start_time,end_time;

int my_connect(char *address,int port,char *protocol,int *sd);


int main(int argc, char **argv){
	int server_port=IMAP_PORT;
	char server_name[256];
	int sd;
	int result;
	char buffer[MAX_INPUT_BUFFER];

	if(argc<2||argc>3){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		printf("IMAP4 plugin for NetSaint\n");
		printf("Copyright (c) 1999 Tom Shields (tom.shields@basswood.com)\n");
		printf("Last Modified: 04-30-1999\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <host_address> [port]\n",argv[0]);
		printf("\n");
		printf("This plugin will attempt to contact the IMAP4 service on the specified host.\n");
		printf("If possible, supply and IP address for the host address, as this will bypass the\n");
		printf("DNS lookup.  If you do not specify a port, the plugin will attempt to connect\n");
		printf("to the host on port %d.\n",server_port);
		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* get the server address */
	strcpy(server_name,argv[1]);

	/* get the port number if one has been specified on the command line */
	if(argc==3)
		server_port=atoi(argv[2]);


	/* try to connect to the host at the given port number */
	result=my_connect(server_name,server_port,IMAP_PROTO,&sd);

	/* we connected, so close connection before exiting */
	if(result==STATE_OK){

		/* watch for the IMAP connection string */
		result=recv(sd,buffer,MAX_INPUT_BUFFER-1,0);

		/* return a WARNING status if we couldn't read any data */
		if(result==-1){
			printf("recv() failed\n");
			result=STATE_WARNING;
		        }

		else{

			/* make sure we find the response we are looking for */
			if(strstr(buffer,IMAP_EXPECT1)==NULL){
				if(server_port==IMAP_PORT)
					printf("Invalid IMAP response received from host\n");
				else
					printf("Invalid IMAP response received from host on port %d\n",server_port);
				result=STATE_WARNING;
			        }

			else{
				time(&end_time);
				printf("IMAP ok - %d second response time\n",(int)(end_time-start_time));
				result=STATE_OK;
			        }
		        }

		/* close the connection */
		send(sd,IMAP_QUIT,strlen(IMAP_QUIT),0);
		close(sd);
		/*unlink(sd);*/
	        }

	return result;
        }


int my_connect(char *host_name,int port,char *protocol,int *sd){
	struct sockaddr_in servaddr;
	struct hostent *hp;
	struct protoent *ptrp;
	int result;

	bzero((char *)&servaddr,sizeof(servaddr));
	servaddr.sin_family=AF_INET;
	servaddr.sin_port=htons(port);

	/* try to bypass using a DNS lookup if this is just an IP address */
	if(!inet_aton(host_name,&servaddr.sin_addr)){

		/* else do a DNS lookup */
		hp=gethostbyname((const char *)host_name);
		if(hp==NULL){
			printf("Invalid host name '%s'\n",host_name);
			return STATE_UNKNOWN;
		        }

		memcpy(&servaddr.sin_addr,hp->h_addr,hp->h_length);
	        }

	/* map TCP transport protocol name to protocol number */
	if(((int)(ptrp=getprotobyname(protocol)))==(int)0){
		printf("Cannot map \"%s\" to protocol number\n",protocol);
		return STATE_UNKNOWN;
	        }

	/* create a socket */
	*sd=socket(PF_INET,SOCK_STREAM,ptrp->p_proto);
	if(*sd<0){
		printf("Socket creation failed\n");
		return STATE_UNKNOWN;
	        }

	/* open a connection */
	time(&start_time);
	result=connect(*sd,(struct sockaddr *)&servaddr,sizeof(servaddr));
	if(result<0){
		switch(errno){  
		case ECONNREFUSED:
			if(port==IMAP_PORT)
				printf("Connection refused by host\n");
			else
				printf("Connection refused at port %d by host\n",port);
			break;
		case ETIMEDOUT:
			printf("Timeout while attempting connection\n");
			break;
		case ENETUNREACH:
			printf("Network is unreachable\n");
			break;
		default:
			printf("Connection refused or timed out\n");
		        }

		return STATE_CRITICAL;
	        }

	return STATE_OK;
        }
